@@ -7,7 +7,8 @@ use warnings;
 use Carp         'confess';
 use Scalar::Util 'blessed';
 
-our $VERSION   = '0.64';
+our $VERSION   = '0.65';
+$VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
 use base 'Class::MOP::Method';
@@ -91,27 +92,27 @@ sub wrap {
         package_name => $params{package_name} || $code->package_name,
         name         => $params{name}         || $code->name,
     );
-    $method->{'%!modifier_table'} = $modifier_table;
+    $method->{'modifier_table'} = $modifier_table;
     $method;
 }
 
 sub get_original_method {
     my $code = shift;
-    $code->{'%!modifier_table'}->{orig};
+    $code->{'modifier_table'}->{orig};
 }
 
 sub add_before_modifier {
     my $code     = shift;
     my $modifier = shift;
-    unshift @{$code->{'%!modifier_table'}->{before}} => $modifier;
-    $_build_wrapped_method->($code->{'%!modifier_table'});
+    unshift @{$code->{'modifier_table'}->{before}} => $modifier;
+    $_build_wrapped_method->($code->{'modifier_table'});
 }
 
 sub add_after_modifier {
     my $code     = shift;
     my $modifier = shift;
-    push @{$code->{'%!modifier_table'}->{after}} => $modifier;
-    $_build_wrapped_method->($code->{'%!modifier_table'});
+    push @{$code->{'modifier_table'}->{after}} => $modifier;
+    $_build_wrapped_method->($code->{'modifier_table'});
 }
 
 {
@@ -132,12 +133,12 @@ sub add_after_modifier {
     sub add_around_modifier {
         my $code     = shift;
         my $modifier = shift;
-        unshift @{$code->{'%!modifier_table'}->{around}->{methods}} => $modifier;
-        $code->{'%!modifier_table'}->{around}->{cache} = $compile_around_method->(
-            @{$code->{'%!modifier_table'}->{around}->{methods}},
-            $code->{'%!modifier_table'}->{orig}->body
+        unshift @{$code->{'modifier_table'}->{around}->{methods}} => $modifier;
+        $code->{'modifier_table'}->{around}->{cache} = $compile_around_method->(
+            @{$code->{'modifier_table'}->{around}->{methods}},
+            $code->{'modifier_table'}->{orig}->body
         );
-        $_build_wrapped_method->($code->{'%!modifier_table'});
+        $_build_wrapped_method->($code->{'modifier_table'});
     }
 }
 
