@@ -6,7 +6,8 @@ use warnings;
 
 use Scalar::Util 'blessed';
 
-our $VERSION   = '0.64';
+our $VERSION   = '0.65';
+$VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
 # introspection
@@ -16,6 +17,10 @@ sub meta {
     Class::MOP::Class->initialize(blessed($_[0]) || $_[0]);
 }
 
+sub _new {
+    shift->meta->new_object(@_);
+}
+
 # RANT:
 # Cmon, how many times have you written 
 # the following code while debugging:
